#!/bin/bash
#
# Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Overrible Variables
ISTIO_VERSION="${ISTIO_VERSION:-1.1.7}"
ISTIO_NAMESPACE="${ISTIO_NAMESPACE:-istio-system}"
REGISTRY="${REGISTRY:-container-registry.oracle.com}"
REGISTRY_NAMESPACE="${REGISTRY_NAMESPACE:-}"
ENABLE_GRANFANA="${ENABLE_GRANFANA:-true}"
ENABLE_PROMETHEUS="${ENABLE_PROMETHEUS:-true}"
# Default these images to the istio namespace
GRAFANA_REGISTRY_NAMESPACE="${GRAFANA_REGISTRY_NAMESPACE:-$REGISTRY_NAMESPACE}"
PROMETHEUS_REGISTRY_NAMESPACE="${PROMETHEUS_REGISTRY_NAMESPACE:-$REGISTRY_NAMESPACE}"
OUTPUT_BASE_DIR="${OUTPUT_BASE_DIR:-oracle}"
HELM_BASE_DIR="${HELM_BASE_DIR:-/etc/istio/artifacts/helm}"

# Validate input
if [[ "${REGISTRY_NAMESPACE}" == "" ]]; then
    echo "Error: The environment variable: REGISTRY_NAMESPACE is not set"
    exit 1
fi

helm --help > /dev/null 2>&1
return_code=$?
if [[ ${return_code} == 127 ]]; then
    echo "[Error systemCheck] Helm is not installed. Please install helm with:"
    echo "yum install helm"
    exit 1
fi

# Script Variables
REGISTRY_FULL="$REGISTRY/$REGISTRY_NAMESPACE"
REGISTRY_FULL_ESCAPED="${REGISTRY}\/${REGISTRY_NAMESPACE}"
GRAFANA_REGISTRY_FULL_ESCAPED="${REGISTRY}\/${GRAFANA_REGISTRY_NAMESPACE}"
PROMETHUES_REGISTRY_FULL_ESCAPED="${REGISTRY}\/${PROMETHEUS_REGISTRY_NAMESPACE}"
ISTIO_OUTPUT_FILE="${OUTPUT_BASE_DIR}/istio.yaml"
ISTIO_CRD_OUTPUT_FILE="${OUTPUT_BASE_DIR}/istio-crds.yaml"

mkdir -p ${OUTPUT_BASE_DIR}

# Gen Kubernete Custom Resource Definitions
helm template ${HELM_BASE_DIR}/istio-init \
    --name istio-init \
    --namespace ${ISTIO_NAMESPACE} \
    --set global.proxy.image=proxyv2 \
    --set global.hub=${REGISTRY_FULL} \
    --set global.tag=${ISTIO_VERSION} > ${ISTIO_CRD_OUTPUT_FILE}

# Gen Kubernete Istio deploy yaml
helm template ${HELM_BASE_DIR}/istio \
    --name istio \
    --namespace ${ISTIO_NAMESPACE} \
    --set global.hub=${REGISTRY_FULL} \
    --set global.tag=${ISTIO_VERSION} \
    --set grafana.enabled=${ENABLE_GRANFANA} \
    --set prometheus.enabled=${ENABLE_PROMETHEUS} > ${ISTIO_OUTPUT_FILE}

# Fix the Grafana Image
first="image: \"grafana\/grafana:6.0.2\""
second="image: \"${GRAFANA_REGISTRY_FULL_ESCAPED}\/grafana:5.4.3\""
sed -i.bak "s|$first|$second|g" ${ISTIO_OUTPUT_FILE}

# Fix the Prometheus Image
first="image: \"docker.io\/prom\/prometheus:v2.3.1\""
second="image: \"${PROMETHUES_REGISTRY_FULL_ESCAPED}\/prometheus:v2.9.1\""
sed -i.bak "s|$first|$second|g" ${ISTIO_OUTPUT_FILE}

# Fix the Busybox Image
first="image: \"busybox:1.30.1\""
second="image: \"oraclelinux:7-slim\""
sed -i.bak "s|$first|$second|g" ${ISTIO_OUTPUT_FILE}

# Rename the Istio Kubectl Image in ISTIO_OUTPUT_FILE and ISTIO_CRD_OUTPUT_FILE
# TODO: We should not rename this image once we create an istio namespace in our registry
first="image: \"${REGISTRY_FULL_ESCAPED}\/kubectl:${ISTIO_VERSION}\""
second="image: \"${REGISTRY_FULL_ESCAPED}\/istio_kubectl:${ISTIO_VERSION}\""
sed -i.bak "s|$first|$second|g" ${ISTIO_OUTPUT_FILE}
sed -i.bak "s|$first|$second|g" ${ISTIO_CRD_OUTPUT_FILE}

# Validate no other images need to be fixed
output=$(grep "image: " ${ISTIO_OUTPUT_FILE} | grep -v ${REGISTRY_FULL} | grep -v ${GRAFANA_REGISTRY_NAMESPACE}| grep -v ${PROMETHEUS_REGISTRY_NAMESPACE} | grep -v oraclelinux:7-slim || true)
if [[ "${output}" != "" ]]; then
    echo -e "ERROR: There are images in the ${ISTIO_OUTPUT_FILE} file that not in the Oracle Registry:\n ${output}"
    exit 1
fi
output=$(grep "image: " ${ISTIO_CRD_OUTPUT_FILE} | grep -v ${REGISTRY_FULL}| grep -v ${GRAFANA_REGISTRY_NAMESPACE}| grep -v ${PROMETHEUS_REGISTRY_NAMESPACE} | grep -v oraclelinux:7-slim || true)
if [[ "${output}" != "" ]]; then
    echo -e "ERROR: There are images in the ${ISTIO_CRD_OUTPUT_FILE} file that not in the Oracle Registry:\n ${output}"
    exit 1
fi

# Validate output files have content
if [ ! -s ${ISTIO_OUTPUT_FILE} ]; then
    echo "ERROR: The ${ISTIO_OUTPUT_FILE} file is expected to have conent"
    exit 1
fi
if [ ! -s ${ISTIO_CRD_OUTPUT_FILE} ]; then
    echo "ERROR: The ${ISTIO_CRD_OUTPUT_FILE} file is expected to have conent"
    exit 1
fi

echo "Successfully Created Istio Install Files :)"
echo ""
echo "Run these commands to DELETE Istio:"
echo "    kubectl delete -f ${ISTIO_OUTPUT_FILE}"
echo "    kubectl delete -f ${ISTIO_CRD_OUTPUT_FILE}"
echo "    kubectl delete ns ${ISTIO_NAMESPACE}"
echo ""
echo "Run these commands to INSTALL Istio:"
echo "    kubectl create ns ${ISTIO_NAMESPACE}"
echo "    kubectl apply -f ${ISTIO_CRD_OUTPUT_FILE}"
echo "    kubectl apply -f ${ISTIO_OUTPUT_FILE}"
